<?php
namespace Teknisa\EST\Services;

use Zeedhi\Framework\DataSource\FilterCriteria;
use Teknisa\Libs\Util\Environment;

use Zeedhi\Framework\DTO;
use Zeedhi\Framework\DataSource\DataSet;
use Teknisa\EST\Factories\Ger27000LocalDeEstoquePorProdutoFactory;

class Ger27000LocalDeEstoquePorProdutoService extends \Teknisa\EST\Controllers\TeknisaCrud {

	protected $environment;
	protected $dataSourceManager;
	protected $factory;

    public function __construct(Environment $environment, $dataSourceManager, $factory){
        $this->environment = $environment;
        $this->dataSourceManager = $dataSourceManager;
        $this->factory = $factory;
    }

    
    public function cadastroAutomaticoLocalProduto($rowObject) {
        $row = (array) $rowObject;
        $dataSourceNameToSave = 'localestoque_produto';
        $produtos = $this->getProdutos($row['CDFILIAL'], $row['P_CDPRODINI'], $row['P_CDPRODFIN'], 5, NULL);
        $rowsUpdate = array();
        foreach ($produtos as $prod) {
            $produto = array_merge($prod, $row);
            unset($produto['__rightActions']);
            unset($produto['__leftActions']);
            unset($produto['P_CDPRODINI']);
            unset($produto['P_CDPRODFIN']);
            $produto = $this->adicionaCampos($produto);
            array_push($rowsUpdate, $this->verificaCamposAlterados($row, $produto));
        }
        $dataSet = new DataSet($dataSourceNameToSave, $rowsUpdate);
        $this->dataSourceManager->persist($dataSet);
    }
    
    protected function getProdutos($CDFILIAL, $P_CDPRODINI, $P_CDPRODFIN, $CDNVPRODUTO, $TODOS) {
        $conditions =  array();
        $filterCriteria = new FilterCriteria('prodfili_arvore', $conditions);
        $paramsProdutos = $params = array(
                    ':CDFILIAL'=>$CDFILIAL,
                    ':CDNVPRODUTO'=>$CDNVPRODUTO,
                    ':P_CDPRODFIN'=>$P_CDPRODFIN,
                    ':P_CDPRODINI'=>$P_CDPRODINI,
                    ':TODOS'=>$TODOS
                );
        $produtos = $this->factory->getProdutos($params);
        $prodReturn = array();
        foreach ($produtos as $prod) {
            $params = array(
                    ':CDFILIAL'=>$CDFILIAL,
                    ':CDPRODUTO'=>$prod['CDPRODUTO']
                );

            if(isset($this->factory->getDadosProduto($params)[0])){
            $produtoComLocal = $this->factory->getDadosProduto($params)[0];
            array_push($prodReturn, $produtoComLocal);
            }
        }
        return $prodReturn;
    }
    
    protected function verificaCamposAlterados($row, $prod) {
        
        $prod["CDALMOXINSPR"] = isset($row["CDALMOXINSPR"]) ? $row["CDALMOXINSPR"] : $prod["CDALMOXINSPR"];
        $prod["DSALMOXINSPR"] = isset($row["DSALMOXINSPR"]) ? $row["DSALMOXINSPR"] : $prod["DSALMOXINSPR"];
        $prod["CDLOCALINSPR"] = isset($row["CDLOCALINSPR"]) ? $row["CDLOCALINSPR"] : $prod["CDLOCALINSPR"];
        $prod["DSLOCALINSPR"] = isset($row["DSLOCALINSPR"]) ? $row["DSLOCALINSPR"] : $prod["DSLOCALINSPR"];
        $prod["CDALMOXPRSEMACA"] = isset($row["CDALMOXPRSEMACA"]) ? $row["CDALMOXPRSEMACA"] : $prod["CDALMOXPRSEMACA"];
        $prod["DSALMOXPRSEMACA"] = isset($row["DSALMOXPRSEMACA"]) ? $row["DSALMOXPRSEMACA"] : $prod["DSALMOXPRSEMACA"];
        $prod["CDLOCALPRSEMACA"] = isset($row["CDLOCALPRSEMACA"]) ? $row["CDLOCALPRSEMACA"] : $prod["CDLOCALPRSEMACA"];
        $prod["DSLOCALPRSEMACA"] = isset($row["DSLOCALPRSEMACA"]) ? $row["DSLOCALPRSEMACA"] : $prod["DSLOCALPRSEMACA"];
        $prod["CDALMOXINSPRPIL"] = isset($row["CDALMOXINSPRPIL"]) ? $row["CDALMOXINSPRPIL"] : $prod["CDALMOXINSPRPIL"];
        $prod["DSALMOXINSPRPIL"] = isset($row["DSALMOXINSPRPIL"]) ? $row["DSALMOXINSPRPIL"] : $prod["DSALMOXINSPRPIL"];
        $prod["CDLOCALINSPRPIL"] = isset($row["CDLOCALINSPRPIL"]) ? $row["CDLOCALINSPRPIL"] : $prod["CDLOCALINSPRPIL"];
        $prod["DSLOCALINSPRPIL"] = isset($row["DSLOCALINSPRPIL"]) ? $row["DSLOCALINSPRPIL"] : $prod["DSLOCALINSPRPIL"];
        $prod["CDALMOXPROD"] = isset($row["CDALMOXPROD"]) ? $row["CDALMOXPROD"] : $prod["CDALMOXPROD"];
        $prod["DSALPROD"] = isset($row["DSALPROD"]) ? $row["DSALPROD"] : $prod["DSALPROD"];
        $prod["CDLOCPROD"] = isset($row["CDLOCPROD"]) ? $row["CDLOCPROD"] : $prod["CDLOCPROD"];
        $prod["DSLOCPROD"] = isset($row["DSLOCPROD"]) ? $row["DSLOCPROD"] : $prod["DSLOCPROD"];
        $prod["CDALMOXPRODPL"] = isset($row["CDALMOXPRODPL"]) ? $row["CDALMOXPRODPL"] : $prod["CDALMOXPRODPL"];
        $prod["DSALPRODPIL"] = isset($row["DSALPRODPIL"]) ? $row["DSALPRODPIL"] : $prod["DSALPRODPIL"];
        $prod["DSLOCPRODPRIL"] = isset($row["DSLOCPRODPRIL"]) ? $row["DSLOCPRODPRIL"] : $prod["DSLOCPRODPRIL"];
        $prod["CDLOCPRODPL"] = isset($row["CDLOCPRODPL"]) ? $row["CDLOCPRODPL"] : $prod["CDLOCPRODPL"];
        $prod["CDALMOXPRACA"] = isset($row["CDALMOXPRACA"]) ? $row["CDALMOXPRACA"] : $prod["CDALMOXPRACA"];
        $prod["DSALMOXPRACA"] = isset($row["DSALMOXPRACA"]) ? $row["DSALMOXPRACA"] : $prod["DSALMOXPRACA"];
        $prod["CDLOCALPRACA"] = isset($row["CDLOCALPRACA"]) ? $row["CDLOCALPRACA"] : $prod["CDLOCALPRACA"];
        $prod["DSLOCALPRACA"] = isset($row["DSLOCALPRACA"]) ? $row["DSLOCALPRACA"] : $prod["DSLOCALPRACA"];
        $prod["CDALMOXPRCLI"] = isset($row["CDALMOXPRCLI"]) ? $row["CDALMOXPRCLI"] : $prod["CDALMOXPRCLI"];
        $prod["DSALMOXPRCLI"] = isset($row["DSALMOXPRCLI"]) ? $row["DSALMOXPRCLI"] : $prod["DSALMOXPRCLI"];
        $prod["CDLOCALPRCLI"] = isset($row["CDLOCALPRCLI"]) ? $row["CDLOCALPRCLI"] : $prod["CDLOCALPRCLI"];
        $prod["DSLOCALPRCLI"] = isset($row["DSLOCALPRCLI"]) ? $row["DSLOCALPRCLI"] : $prod["DSLOCALPRCLI"];
        $prod["CDALMOXARIFE"] = isset($row["CDALMOXARIFE"]) ? $row["CDALMOXARIFE"] : $prod["CDALMOXARIFE"];
        $prod["DSALMOXARIFE"] = isset($row["DSALMOXARIFE"]) ? $row["DSALMOXARIFE"] : $prod["DSALMOXARIFE"];
        $prod["CDLOCALESTOQ"] = isset($row["CDLOCALESTOQ"]) ? $row["CDLOCALESTOQ"] : $prod["CDLOCALESTOQ"];
        $prod["DSLOCALESTOQ"] = isset($row["DSLOCALESTOQ"]) ? $row["DSLOCALESTOQ"] : $prod["DSLOCALESTOQ"];
        $prod["CDALMOXQUAMPPIL"] = isset($row["CDALMOXQUAMPPIL"]) ? $row["CDALMOXQUAMPPIL"] : $prod["CDALMOXQUAMPPIL"];
        $prod["DSALMOXQUAMPPIL"] = isset($row["DSALMOXQUAMPPIL"]) ? $row["DSALMOXQUAMPPIL"] : $prod["DSALMOXQUAMPPIL"];
        $prod["CDLOCALQUAMPPIL"] = isset($row["CDLOCALQUAMPPIL"]) ? $row["CDLOCALQUAMPPIL"] : $prod["CDLOCALQUAMPPIL"];
        $prod["DSLOCALQUAMPPIL"] = isset($row["DSLOCALQUAMPPIL"]) ? $row["DSLOCALQUAMPPIL"] : $prod["DSLOCALQUAMPPIL"];
        $prod["CDALMOXQUAR"] = isset($row["CDALMOXQUAR"]) ? $row["CDALMOXQUAR"] : $prod["CDALMOXQUAR"];
        $prod["DSALMOXQUAR"] = isset($row["DSALMOXQUAR"]) ? $row["DSALMOXQUAR"] : $prod["DSALMOXQUAR"];
        $prod["CDLOCALQUAR"] = isset($row["CDLOCALQUAR"]) ? $row["CDLOCALQUAR"] : $prod["CDLOCALQUAR"];
        $prod["DSLOCALQUAR"] = isset($row["DSLOCALQUAR"]) ? $row["DSLOCALQUAR"] : $prod["DSLOCALQUAR"];
        $prod["CDLOCALREPROV"] = isset($row["CDLOCALREPROV"]) ? $row["CDLOCALREPROV"] : $prod["CDLOCALREPROV"];
        $prod["DSALMOXREPROV"] = isset($row["DSALMOXREPROV"]) ? $row["DSALMOXREPROV"] : $prod["DSALMOXREPROV"];
        $prod["CDLOCALREPROV"] = isset($row["CDLOCALREPROV"]) ? $row["CDLOCALREPROV"] : $prod["CDLOCALREPROV"];
        $prod["DSLOCALREPROV"] = isset($row["DSLOCALREPROV"]) ? $row["DSLOCALREPROV"] : $prod["DSLOCALREPROV"];
        $prod["CDALMOXCONSVND"] = isset($row["CDALMOXCONSVND"]) ? $row["CDALMOXCONSVND"] : $prod["CDALMOXCONSVND"];
        $prod["DSALMOXCONSVND"] = isset($row["DSALMOXCONSVND"]) ? $row["DSALMOXCONSVND"] : $prod["DSALMOXCONSVND"];
        $prod["CDLOCALCONSVND"] = isset($row["CDLOCALCONSVND"]) ? $row["CDLOCALCONSVND"] : $prod["CDLOCALCONSVND"];
        $prod["DSLOCALCONSVND"] = isset($row["DSLOCALCONSVND"]) ? $row["DSLOCALCONSVND"] : $prod["DSLOCALCONSVND"];
        
        return $prod;
    }
    
    protected function adicionaCampos($row) {
        $nrorg = $this->environment->getNrorgTrab();
		$operador = $this->environment->getCdOperador();
		$currentTime = $this->environment->getCurrentTime()->format("d/m/Y H:i:s");
		$row['NRORG'] = $nrorg;
		$row['DTINCLUSAO'] = $currentTime;
		$row['DTULTATU'] = $currentTime;
		$row['CDOPERINCLUSAO'] = $operador;
		$row['CDOPERULTATU'] = $operador;
		$row['IDATIVO'] = 'S';
		$row['NRORGINCLUSAO'] = $nrorg;
		$row['NRORGULTATU'] = $nrorg;
		$row['__is_new'] = false;
		return $row;
    }
}