function MasterDetailController($rootScope, $scope, ScreenService, templateManager, eventEngine, metaDataFactory) {

	var self = this;
	var WIDGET_AFTER_MOVE_ROW_EVENT_NAME = "WidgetAfterMoveRow";
	var WIDGET_ON_CLOSE_SWIPE_EVENT_NAME = "WidgetOnCloseSwipe";

	var eventsToCreate = ["afterMoveRow", "onNewRow", "onOpenDetail", "onCloseSwipe"];

	Object.defineProperty(this, "widget", {
		get : function(){
			return $scope.widget;
		},
		set : function (widget){
			$scope.widget = widget;
		},
		enumerable: true,
		configurable: true
	});

	var formTemplate = this.widget.templateToForm;
	var viewTemplate = this.widget.templateToView;

	self.tabbedWidget = metaDataFactory.widgetFactory({
		"id": "masterDetailTabbedWidget",
		"name": "masterDetailTabbedWidget",
		"container": self.widget.container,
		"parent": self.widget,
		"template": "widget/master_detail/tabbedWidget.html",
		"masterDetailWidget": self.widget,
		"fields": [],
		"events": []
	});

	(self.widget.widgets || []).forEach(function(_widget) {
		_widget.parent = self.tabbedWidget;
		_widget.getParent = function getParent(){
			return self.widget;
		};
	});

	self.tabbedWidget.widgets = self.widget.widgets;

	self.widget.openTab = function openTab(tabName){
		self.tabbedWidget.openTab(tabName);
	};

	function createEvents(events) {
		events.forEach(function(event){
			if (!self.widget[event]) {
				eventEngine.createEvent(self.widget, event);
			}
		});
	}
	createEvents(eventsToCreate);

	var afterMoveRowBoundEventIndex = eventEngine.bindEvent(this.widget, WIDGET_AFTER_MOVE_ROW_EVENT_NAME, setCurrentRowAndOpenSwipe);

	function setCurrentRowAndOpenSwipe() {
		if (!self.widget.inlineEditing && !Util.isClickOnEditableField()) {
			var dataSource = self.widget.dataSource;
			if (dataSource) {
				if (!self.widget.lastCheckedRows) {
					self.widget.lastCheckedRows = 0;
				}
				if ((!dataSource.checkedRows || dataSource.checkedRows.length === 0) &&
					(!(self.widget.lastCheckedRows === 1 && dataSource.checkedRows.length === 0))) {
					self.widget.currentRow.__index = dataSource.data.indexOf(self.widget.currentRow);
					if (self.widget.selectDetailOnMoveRow) {
						self.widget.currentWidget = self.tabbedWidget;
						self.tabbedWidget.currentWidget = self.widget;
					}
					self.widget.childTemplate = self.widget.defaultChildTemplate;
					self.widget.view();

					self.widget.isMasterDetailPopup = true;
					if (!ScreenService.isWidgetAsSwipe(self.tabbedWidget) || self.widget.closeChildSwipes) {
						ScreenService.openSwipe(self.tabbedWidget).then(function() {
							if (self.widget.onOpenDetail) {
								self.widget.onOpenDetail();
							}
						});
					} else {
						if (self.widget.currentWidget) {
							self.widget.currentWidget.activate();
							self.widget.currentWidget.lastActiveWidget = self.widget;
						}
					}
				}
				self.widget.lastCheckedRows = dataSource.checkedRows.length;
			}
		}
	}

	function unbindAfterMoveRowListener() {
		eventEngine.unbindEvent(self.widget, WIDGET_AFTER_MOVE_ROW_EVENT_NAME, afterMoveRowBoundEventIndex);
	}

	var onCloseSwipeBoundEventIndex = eventEngine.bindEvent(this.tabbedWidget, WIDGET_ON_CLOSE_SWIPE_EVENT_NAME, setCurrentWidgetAndList);
	function setCurrentWidgetAndList() {
		self.widget.currentWidget = self.widget;
		self.widget.list();
	}

	function unbindOnCloseSwipeListener() {
		eventEngine.unbindEvent(self.tabbedWidget, WIDGET_ON_CLOSE_SWIPE_EVENT_NAME, onCloseSwipeBoundEventIndex);
	}

	$scope.$on("$destroy", function() {
		unbindOnCloseSwipeListener();
		unbindAfterMoveRowListener();
	});

	var buildChangeModeFunction = function(widget, functionProperty, template) {
		return function() {
			widget.childTemplate = template;
			widget[functionProperty](false, false);
		};
	};

	if(!this.widget.innerEdit){
		this.widget.innerEdit = this.widget.edit;
		this.widget.edit = buildChangeModeFunction(this.widget, "innerEdit", formTemplate);
	}

	if(!this.widget.innerList){
		this.widget.innerList = this.widget.list;
		this.widget.list = buildChangeModeFunction(this.widget, "innerList", formTemplate);
	}

	if(!this.widget.innerView){
		this.widget.innerView = this.widget.view;
		this.widget.view = buildChangeModeFunction(this.widget, "innerView", viewTemplate);
	}

	self.widget.activate();
}